#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <c7[:mt]|e19[:mt|:qt|:vg]|e20[:mt|:qt|:vg]> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_clhep_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    clhep_dir=`cd ${CLHEP_BASE_DIR} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

have_qt_build()
{
    # At this time, QT is only available for slf6 and slf7
    osname=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osname`
    case ${osname} in
      slf6-x86_64)
	have_qt=true
	;;
      slf7-x86_64)
	have_qt=true
	;;
      sl7-x86_64)
	have_qt=true
	;;
      *)
	 echo "ERROR: the QT build of geant4 is only supported for slf6 and slf7"
	 echo "ERROR: attempting to build for ${osname}"
	 exit 1
    esac    
}

make_distribution()
{
    if [ "${maketar}" = "tar" ] 
    then
       if [ -d ${pkgdir}/lib64 ] || [ -d ${pkgdir}/lib ]
       then
	   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
	   exit 0
       fi
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=geant4
origpkgver=v4_10_7_p03
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_15
tarpatchver=`echo ${origpkgver} | cut -f 4 -d"_"`
tarminorver=`echo ${origpkgver} | cut -f 3 -d"_"`
tarmajorver=`echo ${origpkgver} | cut -f 2 -d"_"`
tarbver=`echo ${origpkgver} | cut -f 1-2 -d"_"`
if [[ ${tarpatchver} ]]; then
  tarpatchnum=`echo ${tarpatchver} | sed -e 's/^p0//' | sed -e 's/^b01/0/'`
  tarver=${tarbver}_0${tarminorver}_${tarpatchver}
  g4subdir=Geant4-${tarmajorver}.${tarminorver}.${tarpatchnum}
else
  tarver=${tarbver}_0${tarminorver}
  g4subdir=Geant4-${tarmajorver}.${tarminorver}.0
fi
pkgdotver=`echo ${tarver} | sed -e 's/_/./g' | sed -e 's/^v//'`
#pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=geant${pkgdotver}.tar.gz
#pkgbasedir=`echo ${origpkgver} | sed -e 's/^v4_//'| sed -e 's/_/./g' | sed -e 's/00/0/' | sed -e 's/p0//'`
echo
echo "tarver: ${tarver}"
echo "g4subdir: ${g4subdir}"
echo

get_this_dir

get_ssibuildshims

source define_basics --

make_distribution

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${basequal}" == e1[0245] ]]
then
  cc=gcc
  cxx=g++
  cxxflg="c++14"
elif [[ "${basequal}" == e1[0245]:mt  ]]
then
  cc=gcc
  cxx=g++
  cxxflg="c++14"
  # use the gloabal dynamic thread local storage model
  extra_flags="-DGEANT4_BUILD_MULTITHREADED=ON -DGEANT4_BUILD_TLS_MODEL=global-dynamic"
elif [[ "${basequal}" == e1[0245]:qt  ]]
then
  cc=gcc
  cxx=g++
  cxxflg="c++14"
  have_qt_build
elif [[ "${basequal}" == e1[79] ]] || [[ "${basequal}" == e20 ]] 
then
  cc=gcc
  cxx=g++
  # cxxflg="c++17"
  cxxflg="17"
elif [[ "${basequal}" == e1[79]:mt ]] || [[ "${basequal}" == e20:mt ]]
then
  cc=gcc
  cxx=g++
  # cxxflg="c++17"
  cxxflg="17"
  # use the gloabal dynamic thread local storage model
  extra_flags="-DGEANT4_BUILD_MULTITHREADED=ON -DGEANT4_BUILD_TLS_MODEL=global-dynamic"
elif [[ "${basequal}" == e1[79]:qt ]] || [[ "${basequal}" == e20:qt ]]
then
  cc=gcc
  cxx=g++
  # cxxflg="c++17"
  cxxflg="17"
  have_qt_build
elif [[ "${basequal}" == e1[79]:vg ]] || [[ "${basequal}" == e20:vg ]]
then
  cc=gcc
  cxx=g++
  # cxxflg="c++17"
  cxxflg="17"
  have_vg=true
#  extra_flags="-DCMAKE_PREFIX_PATH=${VECGEOM_FQ_DIR} -DGEANT4_USE_USOLIDS=ON"
elif [[ "${basequal}" == e1[79]:mt:vg ]] || [[ "${basequal}" == e20:mt:vg ]]
then
  cc=gcc
  cxx=g++
  # cxxflg="c++17"
  cxxflg="17"
  have_vg=true
  extra_flags="-DGEANT4_BUILD_MULTITHREADED=ON -DGEANT4_BUILD_TLS_MODEL=global-dynamic"
elif [[ "${basequal}" == c[27] ]]
then
  cc=clang
  cxx=clang++
  # cxxflg="c++17"
  cxxflg="17"
elif [[ "${basequal}" == c[27]:mt ]]
then
  cc=clang
  cxx=clang++
  # cxxflg="c++17"
  cxxflg="17"
  # use the gloabal dynamic thread local storage model
  extra_flags="-DCMAKE_CXX_FLAGS=-D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1 -DGEANT4_BUILD_MULTITHREADED=ON -DGEANT4_BUILD_TLS_MODEL=global-dynamic"
elif [[ "${basequal}" == c[27]:vg ]]
then
  cc=clang
  cxx=clang++
  # cxxflg="c++17"
  cxxflg="17"
  have_vg=true
elif [[ "${basequal}" == c[27]:mt:vg ]]
then
  cc=clang
  cxx=clang++
  # cxxflg="c++17"
  cxxflg="17"
  have_vg=true
  extra_flags="-DCMAKE_CXX_FLAGS=-D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1 -DGEANT4_BUILD_MULTITHREADED=ON -DGEANT4_BUILD_TLS_MODEL=global-dynamic"
else
  echo "ERROR: unrecognized basequal ${basequal}"
  usage
  exit 1
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
# make sure we start from an empty build directory
if [ -d ${blddir} ]; then rm -rf ${blddir}; fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# pull the geant4 datasets
${thisdir}/scripts/getG4DataSets.sh ${product_dir} || ssi_die "ERROR: failed to install datasets"

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "ERROR: fake declare failed"

cd ${sourcedir} || ssi_die "could not cd to ${sourcedir}"
tar xzf ${tardir}/${pkgtarfile} || ssi_die "ERROR: failed to unwind the source."

# apply Fermi patches
# --> patchfile=fermi_geant`echo ${origpkgver} | sed -e 's/^v//'`.patch
# --> patch -b -p0 < ${patchdir}/${patchfile} || ssi_die "ERROR: patch failed."
# bug fix patch
##cd ${sourcedir}/geant${pkgdotver}  || ssi_die "cannot cd to ${sourcedir}/geant${pkgdotver}"
##patch -b -p0 < ${patchdir}/geant4_10.3_and_up.patch || ssi_die "ERROR: patch geant4_10.2.p03_and_up.patch failed."
# c++17 patch
##cd ${sourcedir}
##patch -b -p0 < ${patchdir}/cxx17.patch || ssi_die "ERROR: patch cxx17.patch failed."
# clhep-related patch to geant4.10.7
### cd ${sourcedir}
### patch -b -p0 < ${patchdir}/geant4-10.07-clhep.patch || ssi_die "ERROR: patch geant4-10.07-clhep.patch failed."
### patch -b -p0 < ${patchdir}/geant4-10.07.p01-Mu2e.patch || ssi_die "ERROR: patch geant4-10.07.p01-Mu2e.patch failed."

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed."

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

# Have to wait until after the fake setup to check for QT and set QT_QMAKE_EXECUTABLE
if [[ ${have_qt} ]]
then
  if [ -z ${QT_FQ_DIR} ] 
  then
    echo "ERROR: QT_FQ_DIR is undefined, cannot make a QT build"
    exit 1
  fi
  extra_flags="-DGEANT4_USE_QT=ON -DQT_QMAKE_EXECUTABLE=${QT_FQ_DIR}/bin/qmake"
fi

if [[ ${have_vg} ]]
then
  if [ -z ${VECGEOM_FQ_DIR} ]
  then
    echo "ERROR: VECGEOM_FQ_DIR is undefined, cannot make a VG build"
    exit 1
  fi
  extra_flags="${extra_flags} -DCMAKE_PREFIX_PATH=${VECGEOM_FQ_DIR} -DGEANT4_USE_USOLIDS=ON"   
fi

export GEANT4_SOURCE=${GEANT4_DIR}/source/geant${pkgdotver}
export BLDDIR=${blddir}
GEANT4_J=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

#echo " XERCESCROOT = ${XERCESCROOT}"
#echo " VECGEOM_FQ_DIR = ${VECGEOM_FQ_DIR}"

build_type=RelWithDebInfo
if [ "${extraqual}" == "debug" ]; then
build_type=Debug
fi

cd ${BLDDIR}
env CC=${cc} CXX=${cxx} FC=gfortran cmake -DGEANT4_BUILD_CXXSTD="${cxxflg}" \
                                     -DCMAKE_INSTALL_PREFIX=${GEANT4_FQ_DIR} \
				     -DGEANT4_USE_SYSTEM_CLHEP=ON \
				     -DCLHEP_ROOT_DIR:STRING="${CLHEP_BASE_DIR}" \
				     -DBUILD_STATIC_LIBS=ON \
				     -DGEANT4_USE_OPENGL_X11=ON \
                                     ${extra_flags} \
				     -DGEANT4_USE_GDML=ON \
				     -DXERCESC_ROOT_DIR:STRING="${XERCESCROOT}" \
				     -DCMAKE_BUILD_TYPE=${build_type} \
				     -DOpenGL_GL_PREFERENCE=LEGACY \
				     ${GEANT4_SOURCE}
(( $? == 0 )) || ssi_die "ERROR: cmake failed."
make ${GEANT4_J:+-j ${GEANT4_J}} || ssi_die "ERROR: make failed."
make install  || ssi_die "ERROR: make install failed."

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed."

if [ ! -d ${G4LIB} ]
then
   echo "ERROR: failed to create ${G4LIB}"
   echo "       Something is very wrong"
   exit 1
fi
echo "G4LIB is ${G4LIB}"

# remove build directory from prof distribution
if [ "${extraqual}" = "prof" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
fi

# need to fix paths:
# bin/geant4-config
# lib64/Geant4-xxx/Geant4*.cmake
# share/Geant4-xxx/geant4make/geant4make*sh
get_clhep_dir
fixfile=/tmp/g4fix$$$$.sh
rm -f ${fixfile}

# get list of files to check
shellFile=`ls ${GEANT4_FQ_DIR}/bin/geant4-config ${GEANT4_FQ_DIR}/bin/*sh ${GEANT4_FQ_DIR}/share/${g4subdir}/geant4make/geant4make*sh`
cmakeConfig=`ls ${G4LIB}/${g4subdir}/Geant4*.cmake`

echo "#!/bin/bash" > ${fixfile}
##echo " set -x " >>  ${fixfile}
for file in ${shellFile}
do
  echo "echo \"fixing ${file}\"" >> ${fixfile}
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\${GEANT4_FQ_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${clhep_dir}%\${CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${XERCESCROOT}%\${XERCESCROOT}%g' \\" >> ${fixfile}
  if [[ ${have_qt} ]]
  then
  echo "  | sed 's%${QT_FQ_DIR}%\${QT_FQ_DIR}%g'   \\" >> ${fixfile}    
  fi
  echo "     > ${file} || exit 1" >> $fixfile
  echo "chmod +x  ${file}" >> ${fixfile}
done
for file in ${cmakeConfig}
do
  echo "echo \"fixing ${file}\"" >> ${fixfile}
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${clhep_dir}%\$ENV{CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%g' \\" >> ${fixfile}
  if [[ ${have_qt} ]]
  then
  echo "  | sed 's%${QT_FQ_DIR}%\$ENV{QT_FQ_DIR}%g'   \\" >> ${fixfile}    
  fi
  echo "     > ${file} || exit 1" >> $fixfile
done
chmod +x ${fixfile}
${fixfile} || ssi_die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}

# this must be last
make_distribution

exit 0
